//---------------------------------------------------------------------------//
//  1100101 |_ mov #0, r4         __                                         //
//     11   |_ <0xb380 %5c4>     / _|_ _____ ___                             //
//     0110 |_ 3.50 -> 3.60     |  _\ \ / _ (_-<                             //
//          |_ base# + offset   |_| /_\_\___/__/                             //
//---------------------------------------------------------------------------//

#include <fxos/passes/pcrel.h>
#include <fxos/binary.h>

namespace FxOS {

PcrelPass::PcrelPass(Binary &binary): InstructionPass(binary)
{
}

bool PcrelPass::analyzeInstruction(uint32_t pc, OldInstruction &ci)
{
    AsmInstruction const *i = ci.inst;
    if(!i)
        return true;

    for(int n = 0; n < i->operandCount(); n++) {
        AsmOperand const &arg = i->operand(n);
        Argument &a = ci.args[n];

        if(arg.kind() == AsmOperand::PcRel
            && (i->opsize() == 2 || i->opsize() == 4)) {
            uint32_t addr = (pc & ~(arg.opsize() - 1)) + 4 + arg.disp();
            a.location = RelConstDomain().constant(addr);

            /* Also compute the value. This is sign-extended from 16-bit with
               mov.w. There is no mov.b for this instruction. */
            VirtualSpace &space = m_binary.vspace();
            uint32_t v = -1;

            if(i->opsize() == 2 && space.covers(addr, 2)) {
                v = space.read_i16(addr);
                a.value = RelConstDomain().constant(v);
            }
            if(i->opsize() == 4 && space.covers(addr, 4)) {
                v = space.read_i32(addr);
                a.value = RelConstDomain().constant(v);
            }
        }
        else if(arg.kind() == AsmOperand::PcJump) {
            uint32_t addr = pc + 4 + arg.disp();
            a.location = RelConstDomain().constant(addr);
            a.value = RelConstDomain().constant(addr);
        }
        else if(arg.kind() == AsmOperand::PcAddr) {
            uint32_t addr = (pc & ~3) + 4 + arg.disp();

            /* SH3 manual says that mova uses the target address of the jump
               when in a delay slot. SH4AL-DSP makes it invalid. */
            // if(ci.delayslot) addr = (ci.jmptarget & ~3) + 4 + a.disp;

            a.location = RelConstDomain().constant(addr);
            a.value = RelConstDomain().constant(addr);
        }
    }

    return true;
}

} /* namespace FxOS */
